"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
class Node {
    constructor(name) {
        this.name = name;
        this.data = new Map();
        this.inputs = new Map();
        this.outputs = new Map();
        this.controls = new Map();
        this.position = [0.0, 0.0];
        this.id = Node.nextId();
    }
    static nextId() {
        if (!this.latestId) {
            this.latestId = 1;
        }
        else {
            this.latestId++;
        }
        return this.latestId;
    }
    static resetId() {
        this.latestId = 0;
    }
    add(map, item, key) {
        if (map.has(item.key)) {
            throw new Error(errors_1.EngineError.ItemExistsOnThisNode);
        }
        if (item[key] !== null) {
            throw new Error(errors_1.EngineError.ItemExistsOnSomeNode);
        }
        item[key] = this;
        map.set(item.key, item);
    }
    addControl(control) {
        this.add(this.controls, control, 'parent');
        return this;
    }
    removeControl(control) {
        control.parent = null;
        this.controls.delete(control.key);
    }
    addInput(input) {
        this.add(this.inputs, input, 'node');
        return this;
    }
    removeInput(input) {
        input.removeAllConnections();
        input.node = null;
        this.inputs.delete(input.key);
    }
    addOutput(output) {
        this.add(this.outputs, output, 'node');
        return this;
    }
    removeOutput(output) {
        output.removeAllConnections();
        output.node = null;
        this.outputs.delete(output.key);
    }
    getConnections() {
        const connections = [];
        for (const input of this.inputs.values()) {
            for (const connection of input.connections) {
                connections.push(connection);
            }
        }
        for (const output of this.outputs.values()) {
            for (const connection of output.connections) {
                connections.push(connection);
            }
        }
        return connections;
    }
}
exports.Node = Node;
Node.latestId = 0;
//# sourceMappingURL=node.js.map